import * as React from "react";
import {observer} from "mobx-react-lite";
import {ModelInput, ModelSelect, useModelContext} from "@rx-suite/ui";
import {t} from "@lingui/macro";
import {ScannerRuleModel} from "../../stores/ScannerRuleStore";
import {ScannerRuleAppendParameters} from "./ScannerRuleAppendParameters";
import {ScannerRuleDeleteParameters} from "./ScannerRuleDeleteParameters";
import {useMemo} from "react";

export const ScannerRuleDialogForm: React.FunctionComponent = observer(() => {
    const model = useModelContext() as ScannerRuleModel;
    const ruleParameterRouting = useMemo(() => {
        switch(model.viewModel.operation) {
            case 'append':
            case 'prepend':
                return <ScannerRuleAppendParameters/>
            case 'delete':
                return <ScannerRuleDeleteParameters/>
        }
    }, [model.viewModel.operation])

    return <div>
        <ModelSelect label={t`Operation`} name={"operation"} options={
            [
                { label: t`Append`, value:'append'},
                { label: t`Prepend`, value:'prepend'},
                { label: t`Delete`, value:'delete'}
            ]
        }/>
        {ruleParameterRouting}
        <ModelInput name={'priority'} type={'number'}/>
    </div>
})