import * as React from 'react';
import {BreadModel, makeColumn, makeFormattedDateTimeColumn, ModelInput} from "@rx-suite/ui";
import {ScannerStore} from "../../stores/ScannerStore";
import {ScannerForm} from "./ScannerForm";
import {t} from "@lingui/macro";
import {ScannerActions} from "../../components/ScannerActions";
import {Column} from "react-table";
import {Box} from "@material-ui/core";
import {observer} from "mobx-react-lite";
import {ScannerTypeSelect} from "../../components/form/ScannerTypeSelect";
import {useScannerTypeColumn} from "../../hooks/table_columns/useScannerTypeColumn";

const ScannerBulkFormInput: React.FunctionComponent = () => {
    return <React.Fragment>
        <Box flex={1}>
            <ModelInput name={t`name`}/>
        </Box>
        <Box ml={1} flex={1}>
            <ModelInput name={t`ip_address`}/>
        </Box>
        <Box ml={1} flex={1}>
            <ScannerTypeSelect/>
        </Box>
    </React.Fragment>
};

export const ScannersBreadModel = observer(() => {
    const columns = [
            makeColumn('name', t`Name`, "string"),
            useScannerTypeColumn(),
            makeColumn('ip_address', t`IP Address`, "string"),
            makeFormattedDateTimeColumn('last_read_time', t`Last Read Time`, 'dateTime'),
            makeColumn('last_read_value', t`Last Read Value`, "string")
        ]

    const bulkCreateColumns: Column[] = React.useMemo(() => {
        return [
            {
                Header: t`Name`,
                accessor: 'name',
                id: 'name',
                Cell: (cellInfo: { value: string }) => {
                    return <Box display={'flex'} alignItems={'center'} height={'100%'}>
                        {cellInfo.value}
                    </Box>;
                }
            },
            {
                Header: t`IP Address`,
                accessor: 'ip_address',
                id: 'ip_address',
                Cell: (cellInfo: { value: string }) => {
                    return <Box display={'flex'} alignItems={'center'} height={'100%'}>
                        {cellInfo.value}
                    </Box>;
                }
            },
            {
                Header: t`Type`,
                accessor: 'type',
                id: 'type',
                Cell: (cellInfo: { value: string }) => {
                    return <Box display={'flex'} alignItems={'center'} height={'100%'}>
                        {cellInfo.value}
                    </Box>;
                }
            }
        ];
    }, []);

    return (
        <BreadModel
            store={ScannerStore}
            Form={ScannerForm}
            columns={columns}
            tableProps={
                {
                    actionColumnWidth: 300,
                    customActions: (ScannerModel) => <ScannerActions model={ScannerModel}/>
                }
            }
            bulkCreateProps={
                {
                    bulkCreateInputs: <ScannerBulkFormInput/>,
                    tableColumns: bulkCreateColumns
                }
            }
        />
    )
});
