module ScannerHandler
  class DatalogicMatrix120ScannerHandler < BaseScannerHandler

    def initialize(name, ip)
      super(name, ip, 51236)
    end

    private

    def emit_scan
      @connection_client.send_message(scan_command)

      # scan confirmation
      response = ''
      while (char = @connection_client.read_response(1)[0]) != 0x02.chr do
        response << char
      end

      # strip excess confirmation
      response = ''
      while (char = @connection_client.read_response(1)[0]) == 0x02.chr do
        #no-op
      end
      if char && char != "\n"
        response << char
      end

      # get the real message
      while (char = @connection_client.read_response(1)[0]) != 0x0D.chr do
        response << char
      end

      result = response

      raise FailedToRead unless scan_response_ok?(response)

      result
    end

    def scan_response_ok?(response)
      response != '*NoRead*'
    end

    concerning :Commands do
      private

      def scan_sequence
        [
          {code: :start, bits: 8, literal: 0x02}, # <stx>
          {code: :open_read_gate, bits: 8, literal: "T"},
          {code: :carriage_return, bits: 8, literal: "\r"}, # <CR>
          {code: :line_feed, bits: 8, literal: "\n"}, # <LF>
        ]
      end
    end
  end
end