module ScannerHandler
  class PepperlFuchsIqh1FpV1ScannerHandler < BaseScannerHandler

    def initialize(name, ip, parameters)
      super(name, ip, 10000)

      @parameters = JSON.parse(parameters).symbolize_keys
      @parameters[:start_position] = @parameters[:start_position].to_i unless @parameters[:start_position].nil?
      @parameters[:end_position] = @parameters[:end_position].to_i unless @parameters[:end_position].nil?
    end

    private

    def emit_scan
      response = if @parameters[:tagid] == true
                   scan_from_tagId
                 else
                   scan_from_raw_memory
                 end

      raise FailedToRead if response.nil?

      result = if @parameters[:end_position] && @parameters[:end_position] > 0
         response[@parameters[:start_position]..@parameters[:end_position]]
      else
         response
      end

      result
    end

    def scan_from_tagId
      @connection_client.send_message(scan_command)

      response_body = request_next_bytes

      # pull out the scan contents, discard metadata
      response_body[4..response_body.length - 1].unpack("H*").first
    end

    def scan_from_raw_memory
      whole_response = ''
      offset = 0
      loop do
        @connection_client.send_message(scan_command(offset))

        response_body = request_next_bytes

        partial = response_body[4..response_body.length - 1]

        break if body_empty?(partial)

        whole_response << partial

        offset += 1
      end
      whole_response
    end

    def body_empty?(body)
      bytes = body.bytes
      bytes.all? {|byte| byte == 0}
    end

    def request_next_bytes
      # get confirmation length (-2 for the part we've already consumed)
      confirmation_length = @connection_client.read_response(2)[0].unpack1('H*').to_i(16) - 2

      # discard the confirmation message
      @connection_client.read_response(confirmation_length)

      # step into the real responses, remove the two bytes we've already consumed
      message_length = @connection_client.read_response(2)[0].unpack1('H*').to_i(16) - 2

      @connection_client.read_response(message_length)[0]
    end

    concerning :Commands do
      private

      def scan_sequence
        if @parameters[:tagid] == true
          [
            {code: :telegram_length, bits: 16},
            {code: :command, bits: 8, literal: 0x01},
            {code: :reserved, bits: 4, literal: 0x00},
            {code: :channel, bits: 3, literal: 0b001},
            {code: :toggle_bit, bits: 1}
          ]
        else
          [
            {code: :telegram_length, bits: 16},
            {code: :command, bits: 8, literal: 0x10},
            {code: :words, bits: 4, literal: 0b0010},
            {code: :channel, bits: 3, literal: 0b001},
            {code: :toggle_bit, bits: 1},
            {code: :start_position, bits: 16}
          ]
        end
      end

      def toggle_bit
        @last_scan_toggle ||= true
        @last_scan_toggle = !@last_scan_toggle
        @last_scan_toggle ? 0b1 : 0b0
      end
    end
  end
end