class ScannerManager
  class UnknownScannerHandler < StandardError; end

  class << self
    def handlers
      @handlers ||= {}
    end

    def handler_for_model(model)
      if handlers[model.ip_address].nil?
        handlers[model.ip_address] = case model.type
          when 'WenglorB50'
            ScannerHandler::WenglorB50ScannerHandler.new(model.name, model.ip_address)
          when 'SickClv62x'
            ScannerHandler::SickClv62xScannerHandler.new(model.name, model.ip_address)
          when 'PepperlFuchsIqh1FpV1'
            ScannerHandler::PepperlFuchsIqh1FpV1ScannerHandler.new(model.name, model.ip_address, model.parameters)
          when 'DatalogicMatrix120'
            ScannerHandler::DatalogicMatrix120ScannerHandler.new(model.name, model.ip_address)
          else
            raise UnknownScannerHandler
          end
      end
      handlers[model.ip_address]
    end
  end
end