class AddParametersToScanners < ActiveRecord::Migration[6.1]
  require 'json'

  class Scanner < ActiveRecord::Base
    self.inheritance_column = :_type_disabled
  end

  class WenglorB50 < Scanner
  end

  class DatalogicMatrix120 < Scanner
  end

  class PepperlFuchsIqh1FpV1 < Scanner
  end

  class SickClv62x < Scanner
  end

  def scanner_type_to_parameter_columns
    {
      'SickClv62x' => [],
      'PepperlFuchsIqh1FpV1' => [:whole_memory, :start_position, :end_position, :tagid],
      'DatalogicMatrix120' => [],
      'WenglorB50' => []
    }
  end

  def up
    add_column :scanners, :parameters, :text

    Scanner.connection.schema_cache.clear!
    Scanner.reset_column_information

    Scanner.find_each do |scanner|
      parameters = {}
      parameter_columns = scanner_type_to_parameter_columns[scanner.type]

      next scanner.update(parameters: parameters.to_json) if parameter_columns.empty?

      parameter_columns.each do |column|
        parameters[column] = scanner[column]
      end
      scanner.update(parameters: parameters.to_json)
    end

    remove_column :scanners, :whole_memory
    remove_column :scanners, :tagid
    remove_column :scanners, :start_position
    remove_column :scanners, :end_position
  end

  def down
    add_column :scanners, :whole_memory, :boolean
    add_column :scanners, :start_position, :integer
    add_column :scanners, :end_position, :integer
    add_column :scanners, :tagid, :boolean

    Scanner.connection.schema_cache.clear!
    Scanner.reset_column_information

    Scanner.find_each do |scanner|
      parameters = scanner.parameters ? JSON.parse(scanner.parameters) : nil

      scanner.update(parameters) unless parameters.nil?
    end

    remove_column :scanners, :parameters
  end
end
